class BaseProtocol:
    """
    Base protocol class
    """
    ENCODING = 'utf-8'
    SERVER_CODES = {
        100: 'базовое уведомление',
        101: 'важное уведомление',
        200: 'ОК',
        201: 'объект создан',
        202: 'подтверждение',
        400: 'неправильный запрос/JSON-объект',
        401: 'не авторизован',
        402: 'неправильный логин/пароль',
        403: 'пользователь заблокирован',
        404: 'пользователь/чат отсутствует на сервере',
        409: 'уже имеется подключение с указанным логином',
        410: 'адресат существует, но недоступен (offline)',
        500: 'ошибка сервера'
    }